\name{snow}
\title{Simple Network of Workstations}
\usage{
# Creating and stopping clusters
getMPIcluster()
makeMPIcluster(count, ..., options = defaultClusterOptions)
makePVMcluster(count, ..., options = defaultClusterOptions)
makeSOCKcluster(names, ..., options = defaultClusterOptions)
stopCluster(cl)
stopCluster.default(cl)
stopCluster.MPIcluster(cl)
stopCluster.spawnedMPIcluster(cl)

# Cluster options
defaultClusterOptions
setDefaultClusterOptions(port = 600011, master =  Sys.info()["nodename"],
                         outfile = "/dev/null", rhome = Sys.getenv("R_HOME"),
                         user = Sys.info()["user"], rshcmd = "ssh",
                         rlibs = Sys.getenv("R_LIBS"),
                         scriptdir = .path.package("snow"),
                         rprog = file.path(rhome, "bin", "R"))
addClusterOptions(options, ...)
getClusterOption(name, options = defaultClusterOptions)

# Cluster functions
clusterCall(cl, fun, ...)
clusterApply(cl, x, fun, ...)
clusterEvalQ(cl, expr)

# Parallel functions
parLapply(cl, x, fun, ...)
parRapply(cl, x, fun, ...)
parCapply(cl, x, fun, ...)
parMM(cl, A, B)
parSapply(cl, X, FUN, ..., simplify = TRUE, USE.NAMES = TRUE) 

# Random number generation
clusterSetupSPRNG (cl, seed = round(2^32 * runif(1)),
                            prngkind = "default", para = 0) 

# Internal functions 
enquote(x)
docall(fun, args)
slaveLoop(master)

newSOCKnode(machine = "localhost", ..., options = defaultClusterOptions)
makeSOCKmaster()
closeNode.SOCKnode(node)
sendTag.SOCKnode(node, tag)
recvTag.SOCKnode(node)
sendData.SOCKnode(node, data)
recvData.SOCKnode(node)

newPVMnode(where = "", options = defaultClusterOptions)
makePVMmaster()
sendTag.PVMnode(node, tag)
recvTag.PVMnode(node)
sendData.PVMnode(node, data)
recvData.PVMnode(node)

setMPIcluster(new)
runMPIslave()
newMPInode(rank, comm)
makeMPImaster(comm)
sendTag.MPInode(node, tag)
recvTag.MPInode(node)
sendData.MPInode(node, data)
recvData.MPInode(node)

closeNode(node)
closeNode.default(node)
sendTag(node, tag)
recvTag(node)
sendData(node, data)
recvData(node)

sendNode(con, tag, value = NULL)
postNode(con, tag, value = NULL)
stopNode(n)

sendCall(con, fun, args)

initSprngNode(streamno, nstream, seed, kind, para) 

splitIndices(nx, ncl)
splitList(x, ncl)
splitRows(x, ncl)
splitCols(x, ncl)
}
\alias{makeMPIcluster}
\alias{getMPIcluster}
\alias{setMPIcluster}
\alias{runMPIslave}
\alias{makePVMcluster}
\alias{makeSOCKcluster}
\alias{stopCluster}
\alias{stopCluster.default}
\alias{stopCluster.MPIcluster}
\alias{stopCluster.spawnedMPIcluster}
\alias{defaultClusterOptions}
\alias{setDefaultClusterOptions}
\alias{addClusterOptions}
\alias{getClusterOption}
\alias{clusterCall}
\alias{clusterApply}
\alias{clusterEvalQ}
\alias{parLapply}
\alias{parRapply}
\alias{parCapply}
\alias{parMM}
\alias{parSapply}
\alias{enquote}
\alias{docall}
\alias{slaveLoop}
\alias{newSOCKnode}
\alias{makeSOCKmaster}
\alias{closeNode.SOCKnode}
\alias{sendTag.SOCKnode}
\alias{recvTag.SOCKnode}
\alias{sendData.SOCKnode}
\alias{recvData.SOCKnode}
\alias{newPVMnode}
\alias{makePVMmaster}
\alias{closeNode.PVMnode}
\alias{sendTag.PVMnode}
\alias{recvTag.PVMnode}
\alias{sendData.PVMnode}
\alias{recvData.PVMnode}
\alias{newMPInode}
\alias{makeMPImaster}
\alias{closeNode.MPInode}
\alias{sendTag.MPInode}
\alias{recvTag.MPInode}
\alias{sendData.MPInode}
\alias{recvData.MPInode}
\alias{closeNode}
\alias{closeNode.default}
\alias{sendTag}
\alias{recvTag}
\alias{sendData}
\alias{recvData}
\alias{sendNode}
\alias{postNode}
\alias{stopNode}
\alias{sendCall}
\alias{clusterSetupSPRNG}
\alias{initSprngNode}
\alias{splitIndices}
\alias{splitList}
\alias{splitRows}
\alias{splitCols}
\arguments{
  \item{count}{number of nodes to create}
  \item{names}{character vector of node names}
  \item{options}{cluster options object}
  \item{cl}{cluster object}
  \item{fun}{function or character string naming a function}
  \item{expr}{expression to evaluate}
}
\description{
  Support for simple parallel computing experiments.
}
\details{
  See writeup in \url{http://www.stat.uiowa.edu/~luke/R/cluster/cluster.html}
}
\examples{
  \dontrun{
cl <- makeSOCKcluster(c("localhost","localhost"))
clusterApply(cl, 1:2, get("+"), 3)
  }
}
\keyword{internal}
\author{Luke Tierney}
